@@ -22,6 +22,10 @@ def parse_args():
                         help='Analyze evolocity')
     parser.add_argument('--velocity-score', type=str, default='lm',
                         help='Analyze evolocity')
+    parser.add_argument('--downsample', type=float, default=100.,
+                        help='Percentage to uniformly downsample.')
+    parser.add_argument('--wdownsample', type=float, default=100.,
+                        help='Percentage to weightedly downsampling.')
     args = parser.parse_args()
     return args
 
@@ -138,11 +142,11 @@ def interpret_clusters(adata):
     tprint('Purity, Louvain and subtype: {}'
            .format(np.mean(largest_pct_subtype)))
 
-def plot_umap(adata):
-    sc.pl.umap(adata, color='subtype', save='_gag_subtype.png')
-    sc.pl.umap(adata, color='year', save='_gag_year.png')
-    sc.pl.umap(adata, color='homology', save='_gag_homology.png')
-    sc.pl.umap(adata, color='louvain', save='_gag_louvain.png')
+def plot_umap(adata, namespace='gag'):
+    sc.pl.umap(adata, color='subtype', save=f'_{namespace}_subtype.png')
+    sc.pl.umap(adata, color='year', save=f'_{namespace}_year.png')
+    sc.pl.umap(adata, color='homology', save=f'_{namespace}_homology.png')
+    sc.pl.umap(adata, color='louvain', save=f'_{namespace}_louvain.png')
 
 def seqs_to_anndata(seqs):
     keys = set([ key for seq in seqs for meta in seqs[seq] for key in meta ])
@@ -213,6 +217,10 @@ def evo_gag(args, model, seqs, vocabulary, namespace='gag'):
         namespace += f'_{args.model_name}'
     if args.velocity_score != 'lm':
         namespace += f'_{args.velocity_score}'
+    if args.downsample < 100:
+        namespace += f'_downsample{args.seed}-{args.downsample}'
+    elif args.wdownsample < 100:
+        namespace += f'_wdownsample{args.seed}-{args.wdownsample}'
 
     #############################
     ## Visualize Gag landscape ##
@@ -255,9 +263,44 @@ def evo_gag(args, model, seqs, vocabulary, namespace='gag'):
         ) for subtype in adata.obs['subtype']
     ]
 
+    if args.downsample < 100:
+        np.random.seed(args.seed)
+        n_sample = round(len(adata) * (args.downsample / 100.))
+        rand_idx = np.random.choice(len(adata), size=n_sample, replace=False)
+        adata = adata[rand_idx]
+        sc.pp.neighbors(adata, n_neighbors=40, use_rep='X')
+        sc.tl.louvain(adata, resolution=1.)
+        sc.tl.umap(adata, min_dist=1.)
+
+    elif args.wdownsample < 100:
+        np.random.seed(args.seed)
+        n_sample = round(len(adata) * (args.wdownsample / 100.))
+        # Upweight sequences more recent in time.
+        subtype_weights = {
+            'A': 1., 'AE': 2.,
+            'B': 1., 'C': 1., 'BC': 2.,
+            'D': 1., 'E': 1.,
+            'Other': 0.,
+        }
+        weights = np.array([
+            subtype_weights[subtype] for subtype in adata.obs['simple_subtype']
+        ])
+        weights /= sum(weights)
+        rand_idx = np.random.choice(
+            len(adata), size=n_sample, replace=False, p=weights
+        )
+        adata = adata[rand_idx]
+        sc.pp.neighbors(adata, n_neighbors=40, use_rep='X')
+        sc.tl.louvain(adata, resolution=1.)
+        sc.tl.umap(adata, min_dist=1.)
+
+    if args.downsample < 100 or args.wdownsample < 100:
+        with open(f'target/ev_cache/{namespace}_rand_idx.txt', 'w') as of:
+            of.write('\n'.join([ str(x) for x in rand_idx ]) + '\n')
+
     tprint('Analyzing {} sequences...'.format(adata.X.shape[0]))
     evo.set_figure_params(dpi_save=500)
-    plot_umap(adata)
+    plot_umap(adata, namespace=namespace)
 
     cache_prefix = f'target/ev_cache/{namespace}_knn40'
     try:
@@ -439,7 +482,8 @@ if __name__ == '__main__':
 
         evo_gag(args, model, seqs, vocabulary, namespace=namespace)
 
-        if args.model_name == 'esm1b' and args.velocity_score == 'lm':
+        if args.model_name == 'esm1b' and args.velocity_score == 'lm' and \
+           args.downsample + args.wdownsample == 200:
             tprint('Restrict based on similarity to training:')
             evo_gag(args, model, seqs, vocabulary, namespace='gag_homologous')
 
