@@ -22,6 +22,10 @@ def parse_args():
                         help='Analyze evolocity')
     parser.add_argument('--velocity-score', type=str, default='lm',
                         help='Analyze evolocity')
+    parser.add_argument('--downsample', type=float, default=100.,
+                        help='Percentage to uniformly downsample.')
+    parser.add_argument('--wdownsample', type=float, default=100.,
+                        help='Percentage to weightedly downsampling.')
     args = parser.parse_args()
     return args
 
@@ -270,6 +274,10 @@ def evo_globin(args, model, seqs, vocabulary, namespace='glo'):
         namespace += f'_{args.model_name}'
     if args.velocity_score != 'lm':
         namespace += f'_{args.velocity_score}'
+    if args.downsample < 100:
+        namespace += f'_downsample{args.seed}-{args.downsample}'
+    elif args.wdownsample < 100:
+        namespace += f'_wdownsample{args.seed}-{args.wdownsample}'
 
     #########################
     ## Visualize landscape ##
@@ -303,6 +311,42 @@ def evo_globin(args, model, seqs, vocabulary, namespace='glo'):
                         use_rep='X_onehot')
         sc.tl.umap(adata)
 
+    if args.downsample < 100:
+        np.random.seed(args.seed)
+        n_sample = round(len(adata) * (args.downsample / 100.))
+        rand_idx = np.random.choice(len(adata), size=n_sample, replace=False)
+        adata = adata[rand_idx]
+        sc.pp.neighbors(adata, n_neighbors=50, use_rep='X')
+        sc.tl.louvain(adata, resolution=1.)
+        sc.tl.umap(adata, min_dist=1.)
+
+    elif args.wdownsample < 100:
+        np.random.seed(args.seed)
+        n_sample = round(len(adata) * (args.wdownsample / 100.))
+        subtype_weights = {
+            'neuroglobin': 2.,
+            'cytoglobin': 1.,
+            'myoglobin': 1.,
+            'hemoglobin_alpha': 2.,
+            'hemoglobin_beta': 2.,
+            'other/unlabeled': 1.,
+        }
+        weights = np.array([
+            subtype_weights[subtype] for subtype in adata.obs['globin_type']
+        ])
+        weights /= sum(weights)
+        rand_idx = np.random.choice(
+            len(adata), size=n_sample, replace=False, p=weights
+        )
+        adata = adata[rand_idx]
+        sc.pp.neighbors(adata, n_neighbors=50, use_rep='X')
+        sc.tl.louvain(adata, resolution=1.)
+        sc.tl.umap(adata, min_dist=1.)
+
+    if args.downsample < 100 or args.wdownsample < 100:
+        with open(f'target/ev_cache/{namespace}_rand_idx.txt', 'w') as of:
+            of.write('\n'.join([ str(x) for x in rand_idx ]) + '\n')
+
     tprint('Analyzing {} sequences...'.format(adata.X.shape[0]))
     evo.set_figure_params(dpi_save=500)
     plot_umap(adata, namespace=namespace)
@@ -486,7 +530,8 @@ if __name__ == '__main__':
         tprint('All globin sequences:')
         evo_globin(args, model, seqs, vocabulary, namespace=namespace)
 
-        if args.model_name == 'esm1b' and args.velocity_score == 'lm':
+        if args.model_name == 'esm1b' and args.velocity_score == 'lm' and \
+           args.downsample + args.wdownsample == 200:
             tprint('Restrict based on similarity to training:')
             evo_globin(args, model, seqs, vocabulary, namespace='glo_homologous')
 
